#
# commit-fishing.R, 13 Apr 24
#
# Data from:
# A Study of Bug Resolution Characteristics in Popular Programming Languages
# Jie M. Zhang, Feng Li, Dan Hao, Meng Wang, Hao Tang, Lu Zhang, Mark Harman

library("colorspace")
library("plyr")
# library("speedglm")
library("lme4")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)


add_LOC=function(X)
{
# Project names in Commit have a company prefix, which is not
# present in the CLOC project names
the_proj=grepl(paste0("-", LOC$project[X], "$"), lc$project)
if (sum(the_proj) == 0)
   return(NULL)
proj=lc[the_proj, ]
proj=subset(proj, language=LOC$language[X]) # filter stray matches
proj$LOC=LOC$LOC[X]

return(proj)
}


# headers=c("commitSHA", "parentcommitSHA", "changelines", "additions", "deletions", "committype", "is_Issue", "is_Bug", "change lines file language", "is_Refactor", "files_changed")
headers=c("language", "project", "changelines", "additions", "deletions", "committype", "is_Issue", "is_Bug", "change lines file language", "is_Refactor", "files_changed")

lc=read.csv("lang-chg.csv", as.is=TRUE, header=FALSE)
colnames(lc)=headers

lc$reason=ifelse(lc$is_Bug == "NULL", "", lc$is_Bug)
lc$reason=ifelse(lc$is_Issue == "NULL", lc$reason, paste0(lc$reason, lc$is_Issue))
lc$reason=ifelse(lc$is_Refactor == "NULL", lc$reason, paste0(lc$reason, lc$is_Refactor))

LOC=read.csv("../CLOC/all_cloc.csv", as.is=TRUE)

plot(lc$additions, lc$deletions, log="xy", col=pal_col[2],
	xlab="Added lines", ylab="Deleted lines")

ad_mod=glm(deletions ~ log(additions), data=lc, family=poisson,
				subset=(additions > 0))
summary(ad_mod)

xrange=exp(seq(0, 20, by=0.1))
pred=predict(ad_mod, newdata=data.frame(additions=xrange))

lines(xrange, exp(pred), col=pal_col[1])

dev.copy(dev=png, file="commit-add-delete.png")
dev.off()

# ddply(lc_loc, .(language), function(df) length(unique(df$project)))
#       language V1
# 1            C 58
# 2           C# 56
# 3          C++ 56
# 4           Go 60
# 5         Java 62
# 6   JavaScript 60
# 7  Objective-C 59
# 8          PHP 60
# 9       Python 57
# 10        Ruby 57

# ddply(lc_loc, .(language), function(df) nrow(df))
#       language     V1
# 1            C 474497
# 2           C# 225916
# 3          C++ 406825
# 4           Go 295020
# 5         Java 193589
# 6   JavaScript 304651
# 7  Objective-C  58533
# 8          PHP 388332
# 9       Python 456371
# 10        Ruby 429203

# Merge LOC counts
lc_loc=adply(1:nrow(LOC), .mar=1, add_LOC)

lang_mod=glm(files_changes ~ language, data=lc_loc, family="poisson")
summary(lang_mod)

alang_mod=glm(additions ~ language+log(LOC), data=lc_loc, family="poisson")
summary(alang_mod)
# exp(coef(alang_mod))
#        (Intercept)          languageC#         languageC++          languageGo 
#        4.666872e-07        2.143675e+00        2.018408e+00        1.102452e+00 
#        languageJava  languageJavaScript languageObjective-C         languagePHP 
#        4.331904e-01        1.100267e+00        9.740673e-01        1.559661e+00 
#      languagePython        languageRuby            log(LOC)         committype# 
#        8.912451e-01        3.388908e-01        1.169263e+00        5.695689e+07 
#   committypeComment  committypeDocument       committypefix     committypefix_# 
#        4.757199e+07        9.770036e+08        3.021003e+07        2.777327e+07 
#    committypeFormat    committypeNormal      committypenull 
#        1.292102e+08        5.953377e+07        1.148066e+00 

dlang_mod=glm(deletions ~ language+log(LOC), data=lc_loc, family="poisson")
summary(dlang_mod)
# exp(coef(dlang_mod))
#         (Intercept)          languageC#         languageC++          languageGo 
#        1.809877e-07        2.017114e+00        2.275953e+00        9.295179e-01 
#        languageJava  languageJavaScript languageObjective-C         languagePHP 
#        4.695815e-01        1.091877e+00        1.094786e+00        1.669843e+00 
#      languagePython        languageRuby            log(LOC)         committype# 
#        9.396259e-01        3.284252e-01        1.153363e+00        1.055567e+08 
#   committypeComment  committypeDocument       committypefix     committypefix_# 
#        7.100553e+07        1.782680e+09        5.558058e+07        5.483073e+07 
#    committypeFormat    committypeNormal      committypenull 
#        2.178652e+08        1.193439e+08        1.251996e+00 

# chg_mod=glm(deletions ~ language+log(additions)+is_Bug+is_Issue+committype+project, data=lc, family="poisson",
# chg_mod=speedglm(deletions ~ language+log(additions)+is_Bug, data=lc, family=poisson,
chg_mod=glm(deletions ~ project+log(additions)+is_Bug, data=lc, family=poisson("log"),
							subset=(additions > 0))
summary(chg_mod)

loc_mod=glm(deletions ~ language+log(LOC)+log(additions)+is_Bug, data=lc_loc, family=poisson("log"),
							subset=(additions > 0))
summary(loc_mod)

# Combining everything into one variable, i.e., reason, produced
# a less good fit
# del_mod=glm(deletions ~ language+log(LOC)+reason, data=lc_loc, family=poisson("log"))

del_mod=glm(deletions ~ language+log(LOC)+is_Issue+is_Bug+is_Refactor+committype, data=lc_loc, family=poisson)
summary(del_mod)

add_mod=glm(additions ~ language+log(LOC)+is_Issue+is_Bug+is_Refactor+committype, data=lc_loc, family=poisson)
summary(add_mod)

fc_mod=glm(files_changed ~ language+log(LOC)+is_Issue+is_Bug+is_Refactor+committype, data=lc_loc, family=poisson)
summary(fc_mod)



del_lmod=glmer(deletions ~ language+log(LOC)+(language | project), data=lc_loc, family=poisson)
summary(del_lmod)

add_mod=glm(additions ~ language+log(LOC)+is_Issue+is_Bug+is_Refactor+committype, data=lc_loc, family=poisson)
summary(add_mod)

fc_mod=glm(files_changed ~ language+log(LOC)+is_Issue+is_Bug+is_Refactor+committype, data=lc_loc, family=poisson)
summary(fc_mod)



